=begin
Copyright 2009, Matt666
All Rights Reserved

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

License:   		Free

Author:     	Matthieu NOBLET
Organization:   Matt666
Name:      		Statistics
Version:   		1.0
SU Version:   	6 and 7
Date:      		22/06/2009
Description:   	Shows number of entities sorted by class.
				If nothing is selected before running tool, script select all the model
Usage:      	Menu plugins/Selection/Simple Statistics
				Menu plugins/Selection/Deep Statistics
				Type "Matt.Selection.stats false" in the ruby console to load "Simple Statistics" manually.
				Type "Matt.Selection.stats true" in the ruby console to load "Deep Statistics" manually.
				Context menu/Selection/Simple statistics
				Context menu/Selection/Deep statistics
History:
	1.000   22/06/2009
			First version
=end

require 'sketchup.rb'

if (not file_loaded?('TtMn_selection'))
	$submenu ? organizer=$submenu : organizer=UI.menu('Plugins')
	$TtMn_selection_plugins_submenu = organizer.add_submenu('Selection')
	UI.add_context_menu_handler do |menu|
		$TtMn_selection_contextual_submenu=menu.add_submenu('Selection')
	end
	file_loaded('TtMn_selection')
end

unless file_loaded?(__FILE__)
	file_loaded(__FILE__)
	$TtMn_selection_plugins_submenu.add_item('Deep Statistics'){ Matt_Selection.stats true }
	$TtMn_selection_plugins_submenu.add_item('Simple statistics'){ Matt_Selection.stats false }
	UI.add_context_menu_handler do |menu|
		$TtMn_selection_contextual_submenu.add_item('Simple Statistics'){ Matt_Selection.stats false }
		$TtMn_selection_contextual_submenu.add_item('Deep Statistics'){ Matt_Selection.stats true }
	end
end

module Matt_Selection
# Main program
def self.stats(int)
	model=Sketchup.active_model
	sel=model.selection
	sel.empty? ? sel=model.entities.to_a : sel=sel.to_a
	l=[]
	if int
		sel.each do |e|
			if [Sketchup::ComponentInstance,Sketchup::Group].include? e.class
				l.push e.class
				l=retrieve_sub_ents e,l
			else
				l.push e.class
			end
		end
	else
		sel.each{|e|l.push e.class}
	end
	txt=""
	n=0
	organize_stat_list(l).each do |elt|
		if elt[1]>1
			txt=txt+"\n#{(elt[0].to_s.split'::')[1]}s: #{elt[1]}"
		else
			txt=txt+"\n#{(elt[0].to_s.split'::')[1]}: #{elt[1]}"
		end
		n+=elt[1]
	end
	txt="Number of drawing entities: #{n}\n"+txt if n>1
	if int
		txt=txt+"\n----------\n"
		retrieve_tables.each do |t|
			if t[1]>1
				txt=txt+"\n#{(t[0].to_s.split'::')[1]}s: #{t[1]}"
			else
				txt=txt+"\n#{(t[0].to_s.split'::')[1]}: #{t[1]}"
			end
		end
	end
	puts txt if txt!=""
	UI.messagebox(txt) if txt!=""
	nil
end

# Retruns an array of all sub ents under a component or a group
def self.retrieve_sub_ents(ent,lst)
	model=Sketchup.active_model
	if ent.is_a? Sketchup::ComponentInstance
		ss=ent.definition.entities
	elsif ent.is_a? Sketchup::Group
		ss=ent.entities
	end
	if ss
		ss.each{|e|lst.push e.class}
		ss.select{|e|[Sketchup::ComponentInstance,Sketchup::Group].include? e.class}.each{|e|retrieve_sub_ents(e,lst)}
	end
	return lst.flatten
end

# Returns an array of all tables
# [Sketchup::ComponentDefinition,Sketchup::ComponentDefinition,etc]
def self.retrieve_tables
	l=[]
	model=Sketchup.active_model
	l.push [model.definitions[0].class,model.definitions.count] if model.definitions.count>0
	l.push [model.layers[0].class, model.layers.count] if model.layers.count>0
	l.push [model.materials[0].class, model.materials.count] if model.materials.count>0
	l.push [model.styles[0].class, model.styles.count] if model.styles.count>0
	return l
end

# Returns an array of all tables with number of ents
# [[Sketchup::ComponentDefinition,5],Sketchup::Layer,9]]
def self.organize_stat_list(lst)
	lst=lst.flatten
	lst_f=[]
	while lst.length!=0
		n=lst.length
		e=lst.first
		lst.delete e
		lst_f=[[e,(n-lst.length)]]+lst_f
	end
	return lst_f.sort{|x,y| x[0].to_s<=>y[0].to_s}
end

end # module